// Copyright 2020 Chaos Mesh Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// See the License for the specific language governing permissions and
// limitations under the License.

package command

import (
	"fmt"
	"os"
)

// http://tldp.org/LDP/abs/html/exitcodes.html
const (
	ExitSuccess = iota
	ExitError
	ExitBadConnection
	ExitInterrupted
	ExitIO
	ExitBadArgs = 128
)

// ExitWithError exits with error
func ExitWithError(code int, err error) {
	fmt.Fprintln(os.Stderr, "Error:", err)
	os.Exit(code)
}

// NormalExit exits normally
func NormalExit(msg string) {
	fmt.Fprintln(os.Stdout, msg)
	os.Exit(ExitSuccess)
}

// ExitWithMsg exits with error message
func ExitWithMsg(code int, msg string) {
	fmt.Fprintln(os.Stderr, msg)
	os.Exit(code)
}
